
function AStarFinder(){

	this.options = [
		{ "moveX" :  1, "moveY" : 0 },
		{ "moveX" : -1, "moveY" : 0 }, 
		{ "moveX" :  0, "moveY" : 1 }, 
		{ "moveX" :  0, "moveY" : -1 }
	];

	this.cheapestNode = function(list){
		var cheapestValue;
		var cheapest;

		for (var i = 0; i < list.length; i++) {
			var currentValue = list[i].cost + list[i].heuristic;

			if (cheapest == null || currentValue < cheapestValue){ 
				cheapest = list[i];
				cheapestValue = currentValue;
			}
		};

		return cheapest;
	}


	this.logNode = function(msg, node){
		console.log(msg + " : Node x=" + node.x + " y=" + node.y + "(h=" + node.cost + " + " + node.heuristic + ")");

	}

	this.heuristic = function(from, to){
		return 2 * (Math.abs(from.x - to.x) + Math.abs(from.y - to.y));
	}

	this.exportNodes = function(level, currentNode){
		var nodes = [];
		nodes.push(currentNode);
		var p = currentNode.parent

		while(p){
			nodes.push(p);
			p = p.parent;
		}

		return nodes.reverse();
	}

	this.find = function(level, originCell, destinationCell){

		if (level == null  || originCell == null || destinationCell== null) {
			console.error("Missing element for the pathfinder : ");
			console.log(level);
			console.log(originCell);
			console.log(destinationCell);
		}

		var count=0;
		var openList = [];
		var closedList = [];

		var adj;
		var currentNode;
		
		openList.push(originCell);

		while(openList.length > 0) {
			currentNode = this.cheapestNode(openList);
			
			this.logNode("Processing", currentNode);

			if (currentNode.areSame(destinationCell)){
				this.logNode("FOUND in " + count + " moves !!", currentNode);
				return this.exportNodes(level, currentNode);
			}

			this.logNode("Remove from openlist", currentNode);
			openList.splice(openList.indexOf(currentNode), 1);

			this.logNode("Add to closedlist", currentNode);
			closedList.push(currentNode);

			for (var i = 0; i < this.options.length; i++) {

				var optionX = currentNode.x + this.options[i].moveX;
				var optionY = currentNode.y + this.options[i].moveY;

				if (level.outOfBounds(optionX, optionY)){
					this.logNode("Option is out of bounds", currentNode);
					continue;
				}

				adj = level.map[optionY][optionX];
				adj.cost = currentNode.cost + 1; 
				adj.heuristic = this.heuristic(adj, destinationCell);

				count++;
				this.logNode("Check Adjacents cells", adj);


				if ( openList.indexOf(adj) != -1 || closedList.indexOf(adj) != -1) {
					this.logNode("Adjacent already checked", adj);
					continue;
				}

				this.logNode("Adjacent is new", adj);

				if (adj.type != 3){ //si pas obstacle...
					adj.parent = currentNode;

					this.logNode("Adding Adjacent to openlist", adj);					
					openList.push(adj);
				} 

			};
			


		}


	}





}