function LevelLoader(onLevelLoadedCallback){
	this.onLevelLoadedCallback = onLevelLoadedCallback;
	this.httpRequest = new XMLHttpRequest();
	this.parser = new LevelParser();

	this.onProgress = function(e) {
  		var percentComplete = (e.loaded / e.total) * 100;
  		console.log("Loading game level : " + percentComplete + "%.");
	}

	this.onError = function(e) {
	  console.log("Error " + e.target.status + " while loading.");
	}

	this.onLoad = function(e) {
		var jsonResponse = JSON.parse(this.httpRequest.responseText);
		var level = this.parser.buildLevel(jsonResponse);
		console.timeEnd("loading-level");

		this.onLevelLoadedCallback(level);
	}

	this.loadLevel = function(levelId, onLevelLoadedCallback){
		var d = new Date();
		var url = "levels/" + levelId + "/manifest.json?t=" + d.getTime(); //to avoid caching
		console.log("Loading game level at url : " + url);
		console.time("loading-level");

		var self = this; 
		this.httpRequest.open("GET", url, true);
		this.httpRequest.onprogress = function(e){self.onProgress(e);};
		this.httpRequest.onerror 	= function(e){self.onError(e);};
		this.httpRequest.onload 	= function(e){self.onLoad(e);};
		this.httpRequest.send();
	}


}


function LevelParser(){

	this.buildLevel = function(rawLevel){
		var level = new Level();
		level.id = rawLevel.id;
		level.title = rawLevel.title;
		level.description = rawLevel.description;
		level.colors = rawLevel.colors;

		level.map = [];
		level.plainMap = [];

		var nodeCell;
		var line, col, type;

		console.log("Indexing cells in level...");

		for (line = 0; line < rawLevel.map.length; line++) {
			level.map[line] = [];

			for (col = 0; col < rawLevel.map[line].length; col++) {
				type = rawLevel.map[line][col];

				nodeCell = new NodeCell(col, line);
				nodeCell.type = type;

				level.map[line].push(nodeCell);
				level.plainMap.push(nodeCell);

				if (type == 1){ //Start point
					level.startNodeCell = nodeCell;
				} else if (type == 2) {
					level.endNodeCell = nodeCell;
				}

			};
		};

		console.log("Buiding and Indexing actors in level...");
		var startX = level.startNodeCell.x;
		var startY = level.startNodeCell.y;

		console.log("Big Chief ...");
		level.chief = new Actor(startX, startY, document.getElementById("sprite-a")); // TODO imageManager !
		level.actors.push(level.chief);

		console.log("Minions ...");
		var minion; 
		for (type in rawLevel.minions){
			level.minions[type] = [];
			for (var i = 0; i < rawLevel.minions[type]; i++) {
				minion = new Actor(startX, startY, document.getElementById("sprite-" + type)); // TODO imageManager !
				minion.currentStep = minion.currentStep / 2;
				minion.maxStep = minion.maxStep / 2;
				level.minions[type].push(minion); 
				level.actors.push(minion);
			};
		}

		//console.log("Various actors ...");

		console.log("Done building level.");
		return level;

	}


}